/*
 * $Id: JUnit3Executor.java,v 1.5 2009/12/28 10:08:46 oboehm Exp $
 *
 * Copyright (c) 2009 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * (c)reated 18.12.2009 by oliver (ob@oasd.de)
 */

package patterntesting.concurrent.junit;

import java.lang.reflect.*;

import org.apache.commons.logging.*;

/**
 * This class is responsible for starting a JUnit3 test method.
 * It is responsible for the setup for JUnit 3. The rest is done by
 * the super class.
 *
 * @author oliver
 * @since 18.12.2009
 */
public class JUnit3Executor extends JUnitExecutor {

    private static Log log = LogFactory.getLog(JUnit3Executor.class);

    /**
     * @param clazz the class with the test methods (JUnit 4)
     */
    public JUnit3Executor(Class<?> clazz) {
        super(clazz);
        if (this.isRunParallelEnabled()) {
            this.setupMethods();
            this.recordTestMethods();
        }
    }

    /**
     * Here we look after public void methods with "test" as prefix and with no
     * arguments. We look here also for the setUp and tearDown method.
     */
    private void setupMethods() {
        Method[] methods = this.getTestClass().getDeclaredMethods();
        for (int i = 0; i < methods.length; i++) {
            Method method = methods[i];
            int mod = method.getModifiers();
            if (Modifier.isPrivate(mod) || Modifier.isStatic(mod)) {
                continue;
            }
            if (method.getParameterTypes().length > 0) {
                continue;
            }
            Class<?> returnType = method.getReturnType();
            if (!returnType.toString().equalsIgnoreCase("void")) {
                continue;
            }
            String name = method.getName();
            if (name.startsWith("test")) {
                if (Modifier.isPublic(mod)) {
                    results.put(name, new Result(method));
                } else {
                    log.warn(method + " isn't public");
                }
            } else if (name.equals("setUp")) {
                this.setupMethod = method;
            } else if (name.equals("tearDown")) {
                this.teardownMethod = method;
            }
        }
    }

}
